@@ -574,7 +574,7 @@ class TestWSGITask(unittest.TestCase):
         environ = inst.get_environment()
         self.assertEqual(environ['PATH_INFO'], '/bar')
         self.assertEqual(environ['SCRIPT_NAME'], '/foo')
-        
+
     def test_get_environ_with_url_prefix_hit(self):
         inst = self._makeOne()
         inst.channel.server.adj.url_prefix = '/foo'
@@ -593,16 +593,28 @@ class TestWSGITask(unittest.TestCase):
             'CONTENT_TYPE': 'abc',
             'CONTENT_LENGTH': '10',
             'X_FOO': 'BAR',
+            'CONNECTION': 'close',
         }
         request.query = 'abc'
         inst.request = request
         environ = inst.get_environment()
+
+        # nail the keys of environ
+        self.assertEqual(sorted(environ.keys()), [
+            'CONTENT_LENGTH', 'CONTENT_TYPE', 'HTTP_CONNECTION', 'HTTP_X_FOO',
+            'PATH_INFO', 'QUERY_STRING', 'REMOTE_ADDR', 'REQUEST_METHOD',
+            'SCRIPT_NAME', 'SERVER_NAME', 'SERVER_PORT', 'SERVER_PROTOCOL',
+            'SERVER_SOFTWARE', 'wsgi.errors', 'wsgi.file_wrapper', 'wsgi.input',
+            'wsgi.multiprocess', 'wsgi.multithread', 'wsgi.run_once',
+            'wsgi.url_scheme', 'wsgi.version'])
+
         self.assertEqual(environ['REQUEST_METHOD'], 'GET')
         self.assertEqual(environ['SERVER_PORT'], '80')
         self.assertEqual(environ['SERVER_NAME'], 'localhost')
         self.assertEqual(environ['SERVER_SOFTWARE'], 'waitress')
         self.assertEqual(environ['SERVER_PROTOCOL'], 'HTTP/1.0')
         self.assertEqual(environ['SCRIPT_NAME'], '')
+        self.assertEqual(environ['HTTP_CONNECTION'], 'close')
         self.assertEqual(environ['PATH_INFO'], '/')
         self.assertEqual(environ['QUERY_STRING'], 'abc')
         self.assertEqual(environ['REMOTE_ADDR'], '127.0.0.1')
@@ -629,7 +641,7 @@ class TestErrorTask(unittest.TestCase):
         from waitress.task import ErrorTask
         return ErrorTask(channel, request)
 
-    def test_execute(self):
+    def test_execute_http_10(self):
         inst = self._makeOne()
         inst.execute()
         lines = filter_lines(inst.channel.written)
@@ -644,6 +656,55 @@ class TestErrorTask(unittest.TestCase):
         self.assertEqual(lines[7], b'body')
         self.assertEqual(lines[8], b'(generated by waitress)')
 
+    def test_execute_http_11(self):
+        inst = self._makeOne()
+        inst.version = '1.1'
+        inst.execute()
+        lines = filter_lines(inst.channel.written)
+        self.assertEqual(len(lines), 8)
+        self.assertEqual(lines[0], b'HTTP/1.1 432 Too Ugly')
+        self.assertEqual(lines[1], b'Content-Length: 43')
+        self.assertEqual(lines[2], b'Content-Type: text/plain')
+        self.assertTrue(lines[3])
+        self.assertEqual(lines[4], b'Server: waitress')
+        self.assertEqual(lines[5], b'Too Ugly')
+        self.assertEqual(lines[6], b'body')
+        self.assertEqual(lines[7], b'(generated by waitress)')
+
+    def test_execute_http_11_close(self):
+        inst = self._makeOne()
+        inst.version = '1.1'
+        inst.request.headers['CONNECTION'] = 'close'
+        inst.execute()
+        lines = filter_lines(inst.channel.written)
+        self.assertEqual(len(lines), 9)
+        self.assertEqual(lines[0], b'HTTP/1.1 432 Too Ugly')
+        self.assertEqual(lines[1], b'Connection: close')
+        self.assertEqual(lines[2], b'Content-Length: 43')
+        self.assertEqual(lines[3], b'Content-Type: text/plain')
+        self.assertTrue(lines[4])
+        self.assertEqual(lines[5], b'Server: waitress')
+        self.assertEqual(lines[6], b'Too Ugly')
+        self.assertEqual(lines[7], b'body')
+        self.assertEqual(lines[8], b'(generated by waitress)')
+
+    def test_execute_http_11_keep(self):
+        inst = self._makeOne()
+        inst.version = '1.1'
+        inst.request.headers['CONNECTION'] = 'keep-alive'
+        inst.execute()
+        lines = filter_lines(inst.channel.written)
+        self.assertEqual(len(lines), 8)
+        self.assertEqual(lines[0], b'HTTP/1.1 432 Too Ugly')
+        self.assertEqual(lines[1], b'Content-Length: 43')
+        self.assertEqual(lines[2], b'Content-Type: text/plain')
+        self.assertTrue(lines[3])
+        self.assertEqual(lines[4], b'Server: waitress')
+        self.assertEqual(lines[5], b'Too Ugly')
+        self.assertEqual(lines[6], b'body')
+        self.assertEqual(lines[7], b'(generated by waitress)')
+
+
 class DummyError(object):
     code = '432'
     reason = 'Too Ugly'
